@@ -24,13 +24,16 @@
 #include <dbus/dbus-glib-lowlevel.h>
 #include <dbus/dbus-glib.h>
 
+#include "nm-glib-compat.h"
 #include "nm-manager.h"
 #include "nm-utils.h"
 #include "nm-dbus-manager.h"
 #include "nm-vpn-manager.h"
 #include "nm-device-interface.h"
 #include "nm-device-private.h"
+#include "nm-device-ethernet.h"
 #include "nm-device-wifi.h"
+#include "nm-serial-device.h"
 #include "NetworkManagerSystem.h"
 #include "nm-properties-changed-signal.h"
 #include "nm-setting-connection.h"
@@ -76,7 +79,8 @@ static void connection_added_default_handler (NMManager *manager,
 
 static void hal_manager_udi_added_cb (NMHalManager *hal_mgr,
                                       const char *udi,
-                                      const char *type_name,
+                                      const char *originating_device,
+                                      gpointer general_type_ptr,
                                       NMDeviceCreatorFn creator_fn,
                                       gpointer user_data);
 
@@ -95,7 +99,8 @@ static void system_settings_properties_changed_cb (DBusGProxy *proxy,
                                                    GHashTable *properties,
                                                    gpointer user_data);
 
-#define SSD_POKE_INTERVAL 120000
+#define SSD_POKE_INTERVAL 120
+#define ORIGDEV_TAG "originating-device"
 
 typedef struct {
 	DBusGMethodInvocation *context;
@@ -1155,6 +1160,29 @@ nm_manager_get_device_by_udi (NMManager *manager, const char *udi)
 	return NULL;
 }
 
+static NMDevice *
+nm_manager_get_device_by_originating_device (NMManager *manager, const char *od)
+{
+	GSList *iter;
+
+	for (iter = NM_MANAGER_GET_PRIVATE (manager)->devices; iter; iter = iter->next) {
+		const char *candidate_od = g_object_get_data (G_OBJECT (iter->data), ORIGDEV_TAG);
+
+		if (candidate_od && !strcmp (candidate_od, od))
+			return NM_DEVICE (iter->data);
+	}
+	return NULL;
+}
+
+static void
+nm_manager_set_originating_device (NMDevice *device, const char *originating_device)
+{
+	g_return_if_fail (device != NULL);
+	g_return_if_fail (originating_device != NULL);
+
+	g_object_set_data_full (G_OBJECT (device), ORIGDEV_TAG, g_strdup (originating_device), g_free);
+}
+
 static gboolean
 nm_manager_udi_is_managed (NMManager *self, const char *udi)
 {
@@ -1389,7 +1417,7 @@ poke_system_settings_daemon_cb (gpointer user_data)
 
 out:
 	/* Reschedule the poke */
-	priv->poke_id = g_timeout_add (SSD_POKE_INTERVAL, poke_system_settings_daemon_cb, (gpointer) manager);
+	priv->poke_id = g_timeout_add_seconds (SSD_POKE_INTERVAL, poke_system_settings_daemon_cb, (gpointer) manager);
 
 	return FALSE;
 }
@@ -1646,14 +1674,16 @@ next:
 static void
 hal_manager_udi_added_cb (NMHalManager *hal_mgr,
                           const char *udi,
-                          const char *type_name,
+                          const char *originating_device,
+                          gpointer general_type_ptr,
                           NMDeviceCreatorFn creator_fn,
                           gpointer user_data)
 {
 	NMManager *self = NM_MANAGER (user_data);
 	NMManagerPrivate *priv = NM_MANAGER_GET_PRIVATE (self);
 	GObject *device;
-	const char *iface;
+	const char *iface, *driver;
+	GType general_type = GPOINTER_TO_SIZE (general_type_ptr);
 
 	if (priv->sleeping)
 		return;
@@ -1662,10 +1692,18 @@ hal_manager_udi_added_cb (NMHalManager *hal_mgr,
 	if (nm_manager_get_device_by_udi (self, udi))
 		return;
 
-	device = creator_fn (hal_mgr, udi, nm_manager_udi_is_managed (self, udi));
+	/* Only ignore multiple ports for serial devices */
+	if (general_type == NM_TYPE_SERIAL_DEVICE) {
+		if (nm_manager_get_device_by_originating_device (self, originating_device))
+			return;
+	}
+
+	device = creator_fn (hal_mgr, udi, originating_device, nm_manager_udi_is_managed (self, udi));
 	if (!device)
 		return;
 
+	nm_manager_set_originating_device (NM_DEVICE (device), originating_device);
+
 	priv->devices = g_slist_append (priv->devices, device);
 
 	g_signal_connect (device, "state-changed",
@@ -1685,7 +1723,18 @@ hal_manager_udi_added_cb (NMHalManager *hal_mgr,
 	}
 
 	iface = nm_device_get_iface (NM_DEVICE (device));
-	nm_info ("Found new %s device '%s'.", type_name, iface);
+	driver = nm_device_get_driver (NM_DEVICE (device));
+	if (!driver)
+		driver = "unknown";
+
+	if (general_type == NM_TYPE_DEVICE_ETHERNET)
+		nm_info ("(%s): new Ethernet device (driver: '%s')", iface, driver);
+	else if (general_type == NM_TYPE_DEVICE_WIFI)
+		nm_info ("(%s): new 802.11 WiFi device (driver: '%s')", iface, driver);
+	else if (general_type == NM_TYPE_SERIAL_DEVICE)
+		nm_info ("(%s): new Modem device (driver: '%s')", iface, driver);
+	else
+		g_assert_not_reached ();
 
 	dbus_g_connection_register_g_object (nm_dbus_manager_get_connection (priv->dbus_mgr),
 								  nm_device_get_udi (NM_DEVICE (device)),
@@ -2028,6 +2077,14 @@ is_user_request_authorized (NMManager *manager,
 		goto out;
 	}
 
+	/* Let root activate anything.
+	 * FIXME: use a PolicyKit permission instead
+	 */
+	if (0 == sender_uid) {
+		success = TRUE;
+		goto out;
+	}
+
 	service_name = dbus_g_proxy_get_bus_name (priv->user_proxy);
 	if (!service_name) {
 		g_set_error (error, NM_MANAGER_ERROR,
@@ -2134,7 +2191,7 @@ impl_manager_activate_connection (NMManager *manager,
 		info->scope = scope;
 		info->connection_path = g_strdup (connection_path);
 		info->specific_object_path = g_strdup (real_sop);
-		info->timeout_id = g_timeout_add (5000, wait_for_connection_expired, manager);
+		info->timeout_id = g_timeout_add_seconds (5, wait_for_connection_expired, manager);
 
 		// FIXME: should probably be per-device, not global to the manager
 		NM_MANAGER_GET_PRIVATE (manager)->pending_connection_info = info;
